/*
 * update.h: TVM2VDR plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#ifndef __UPDATE_H
#define __UPDATE_H

#include <curl/curl.h>
#include <curl/types.h>
#include <curl/easy.h>

#include <libxslt/transform.h>
#include <libxslt/xsltutils.h>
#include <libexslt/exslt.h>

#ifdef USE_IMAGEMAGICK
#include <magick/api.h>
#ifdef VERSION // Workaround for old and buggy imagemagick
#undef VERSION
#endif
#endif

#include <string>
#include <iostream>
#include <fstream>
#include <sstream>

#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>

#include "i18n.h"
#include "config.h"
#include "channelmap.h"

using namespace std;


#define TVM2VDR_USERAGENT "libcurl-agent/1.0"

struct MemoryStruct {
    char   *memory;
    size_t  size;
};


class cUpdate : public cThread {
private:
    bool active;
    struct MemoryStruct data;
    cChannelMap *chanmap;
    char *xmldir;
    char *epgimagedir;
    xsltStylesheetPtr pxsltStylesheet;
    xsltStylesheetPtr pxsltStylesheetGetPictures;
    int withutf8;
    int isupdatefile;

	
    void Action(void);
    int  DownloadData(const char *url);
    void unscramble();
    bool MakeEPGDataUpdate(void);
    void ProcessImportedFile(const int tvmid, const char *outfilename);
    void GetPictures (const char *xmlfile);
    void RemoveObsoleteFiles(void);
    void RemovePictures (const char *xmlfile);

    xmlDocPtr ParseXMLFile (const char *xmlfile, xsltStylesheetPtr pxsltStylesheet, const char **params);

public:
    cUpdate();
    ~cUpdate();
    void StartUpdate();
    int ReloadMaps();
    void LoadXSLT();
    int loglevel;
    bool fullupdate;
    bool isActive() { return active; };
    void Stop() { active = false; };
};

#endif //__UPDATE_H
